import java.io.*;
import java.util.*;

public class DemoIO
{
    public static void main(String[] args) throws IOException, ClassNotFoundException {
        List<Employee> employees = new ArrayList(List.of(
            new Employee(2000, "Karel Omacka"),
            new Employee(2001, "Josef Novak")
        ));
        
        backupBin(new File("out.bin"), employees);
        employees.clear();
        employees.addAll(recoverBin(new File("out.bin")));
        System.out.println(employees);
    }
    
    public static void backupTxt(File file, List<Employee> emp) throws IOException {
        try (FileWriter w = new FileWriter(file)) {
            backupTxt(w, emp);
        }
    }
    
    public static void backupTxt(Writer w, List<Employee> emp) throws IOException {
        BufferedWriter writer = new BufferedWriter(w);
        for (Employee e: emp) {
            writer.write(e.toString());
            writer.newLine();
        }
    }
    
    public static List<Employee> recoverTxt(Reader r) throws IOException {
        List<Employee> emp = new ArrayList<>();
        
        BufferedReader reader = new BufferedReader(r); 
        String line = reader.readLine();
        while (line != null) {
            String[] rec = line.split(":", 2);
            if (rec.length != 2) {
                throw new IOException("Wrong format " + line);
            }
            String name = rec[0];
            try {
                int id = Integer.parseInt(rec[1]);
                emp.add(new Employee(id, name));
                line = reader.readLine();
            } catch(NumberFormatException ex) {
                throw new IOException("Wrong format " + line, ex);
            }            
        }
        
        return emp;
    }
    
    public static List<Employee> recoverTxt(File file) throws IOException {
        try (FileReader reader = new FileReader(file)) {
            return recoverTxt(reader);
        }
    }

    public static void backupBin(OutputStream os, List<Employee> emp) throws IOException {
        ObjectOutputStream oos = new ObjectOutputStream(os);
        for (Employee e: emp) {
            oos.writeInt(e.getId());
            oos.writeObject(e.getName());
        }
    }
    
    public static void backupBin(File file, List<Employee> emp) throws IOException {
        try (FileOutputStream fos = new FileOutputStream(file)) {
            backupBin(fos, emp);
        }
    }

    public static List<Employee> recoverBin(InputStream in) throws IOException, ClassNotFoundException {
        List<Employee> emp = new ArrayList<>();
        
        ObjectInputStream ois = new ObjectInputStream(in); 
        
        emp.add(new Employee(ois.readInt(),(String) ois.readObject()));
        emp.add(new Employee(ois.readInt(),(String) ois.readObject()));
        
        return emp;
    }
    
    public static List<Employee> recoverBin(File file) throws IOException, ClassNotFoundException {
        try (FileInputStream fis = new FileInputStream(file)) {
            return recoverBin(fis);
        }
    }
}







